
# django-auto-webassets - Automatic webassets javascript bundles for django
# Copyright (C) 2019 Thomas Hartmann <thomas.hartmann@th-ht.de>
#
# This file is part of django-auto-webassets.
#
# django-auto-webassets is free software: you can redistribute it and/or
# modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-auto-webassets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with django-auto-webassets.  If not, see
# <http://www.gnu.org/licenses/>.


from django_assets import Bundle, register

css = Bundle('css/test_css.css',
             output='main.css',
             filters='cssrewrite')

register('css_main', css)




# django-auto-webassets - Automatic webassets javascript bundles for django
# Copyright (C) 2019 Thomas Hartmann <thomas.hartmann@th-ht.de>
#
# This file is part of django-auto-webassets.
#
# django-auto-webassets is free software: you can redistribute it and/or
# modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-auto-webassets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with django-auto-webassets.  If not, see
# <http://www.gnu.org/licenses/>.

import importlib
import inspect
import logging
import os.path
import sys

from django.core.management.base import BaseCommand, CommandError
from django.urls import URLPattern, URLResolver, get_resolver
from django.conf import settings
from django_assets.env import get_env
from webassets.script import (CommandError as AssetCommandError,
                              GenericArgparseImplementation)

from ...utils.webassetsrjs import WebassetsRjsLoader
from ...views.mixins import AutoWebassetsJSMixin


class Command(BaseCommand):
    """Implement the webassetsrequirejs command."""

    help = 'Build assets defined by the AutoWebassetsJSMixin mixin.'
    requires_system_checks = True

    def handle(self, *args, **options):
        """
        Build all assets defined by django_auto_webassets.

        Parameters
        ----------
        args : list
            Not used
        options : dict
            Not used

        """
        log = logging.getLogger('th_utils-assets')
        log.setLevel({0: logging.WARNING,
                      1: logging.INFO,
                      2: logging.DEBUG}[int(options.get('verbosity', 1))])
        log.addHandler(logging.StreamHandler())

        dummy = get_resolver().url_patterns
        view_class_names = self._parse_urls(dummy).keys()
        for cur_class_name in view_class_names:
            (cur_package, cur_module) = cur_class_name.rsplit('.', 1)
            try:
                cur_class = getattr(importlib.import_module(cur_package),
                                    cur_module)
                if inspect.isclass(cur_class) and \
                        issubclass(cur_class, AutoWebassetsJSMixin):
                    cur_class()
            except ModuleNotFoundError:
                pass

        get_env().add(*[b for b in self._load_from_templates()
                        if not b.is_container])

        prog = "%s assets" % os.path.basename(sys.argv[0])
        impl = GenericArgparseImplementation(
            env=get_env(), log=log, no_global_options=True, prog=prog)
        try:
            # The webassets script runner may either return None on success (so
            # map that to zero) or a return code on build failure (so raise
            # a Django CommandError exception when that happens)
            if settings.ASSETS_DEBUG:
                return

            impl.run_with_argv(('clean',)) or 0
            retval = impl.run_with_argv(('build',)) or 0
            if retval != 0:
                raise CommandError('The webassets build script exited with '
                                   'a non-zero exit code (%d).' % retval)
        except AssetCommandError as e:
            raise CommandError(e)

    def _load_from_templates(self):
        # Using the Django loader
        bundles = WebassetsRjsLoader().load_bundles()

        return bundles

    def _parse_urls(self, patterns):
        all_patterns = {}
        if isinstance(patterns, list):
            for cur_pattern in patterns:
                all_patterns.update(self._parse_urls(cur_pattern))

        if isinstance(patterns, URLResolver):
            cur_namespace = patterns.namespace
            all_items = {}
            for cur_pattern in patterns.url_patterns:
                all_items.update(self._parse_urls(cur_pattern))

            for cur_key, cur_item in all_items.items():
                if not cur_namespace:
                    all_items[cur_key] = '%s' % (cur_item,)
                else:
                    all_items[cur_key] = '%s:%s' % (cur_namespace, cur_item)

            return all_items

        if isinstance(patterns, URLPattern):
            return {patterns.lookup_str: patterns.name}

        return all_patterns




# django-auto-webassets - Automatic webassets javascript bundles for django
# Copyright (C) 2019 Thomas Hartmann <thomas.hartmann@th-ht.de>
#
# This file is part of django-auto-webassets.
#
# django-auto-webassets is free software: you can redistribute it and/or
# modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-auto-webassets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with django-auto-webassets.  If not, see
# <http://www.gnu.org/licenses/>.

import inspect
import uuid
import os.path

from ..settings import DJANGO_AUTO_WEBASSETS_Settings
from ..utils.webassetsrjs import make_bundle


class AutoWebassetsJSMixin:
    """View Mixin to specify view-specific javascript bundle."""

    webasset_bundle_name = None
    webasset_js_file = None

    @classmethod
    def _get_complete_classname(cls):
        return '.'.join([inspect.getmodule(cls).__name__, cls.__name__])

    @classmethod
    def _get_webasset_output_name(cls):
        these_settings = DJANGO_AUTO_WEBASSETS_Settings()

        if these_settings.WEBASSETS_SCRAMBLE_OUT_JS:
            return '.'.join([str(uuid.uuid4()), 'js'])
        else:
            return '.'.join([cls._get_complete_classname(), 'js'])

    @classmethod
    def get_webasset_bundle_name(cls):
        """Return the name of the webasset bundle."""
        if cls.webasset_bundle_name:
            return cls.webasset_bundle_name
        else:
            return cls._get_complete_classname()

    @classmethod
    def get_webasset_js_file(cls):
        """Return the filename of the javascript file."""
        if cls.webasset_js_file:
            return cls.webasset_js_file

    @property
    def full_webasset_path(self):
        """Return the full path to the javascript file."""
        these_settings = DJANGO_AUTO_WEBASSETS_Settings()
        js_path = these_settings.WEBASSETS_JS_FOLDER

        if self.get_webasset_js_file():
            return os.path.join(js_path, self.get_webasset_js_file())
        else:
            return None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.full_webasset_path:
            make_bundle(self.full_webasset_path,
                        self._get_webasset_output_name(),
                        self.get_webasset_bundle_name())




# django-auto-webassets - Automatic webassets javascript bundles for django
# Copyright (C) 2019 Thomas Hartmann <thomas.hartmann@th-ht.de>
#
# This file is part of django-auto-webassets.
#
# django-auto-webassets is free software: you can redistribute it and/or
# modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-auto-webassets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with django-auto-webassets.  If not, see
# <http://www.gnu.org/licenses/>.

import inspect

from django import template

import django_assets.env
from ..views.mixins import AutoWebassetsJSMixin

from ..utils.webassetsrjs import make_bundle

register = template.Library()


def _get_complete_classname(cls):
    return '.'.join([inspect.getmodule(cls).__name__, cls.__name__])


@register.inclusion_tag('webassets_tag.html', takes_context=True)
def webassets_js(context, default_js=None):
    """Template Tag for auto webassets."""
    if 'view' in context:
        this_view = context['view']
    elif 'page' in context:
        this_view = context['page']
    else:
        return {'full_class_name': None}

    view_class = this_view.__class__
    view_class_name = _get_complete_classname(view_class)

    if isinstance(this_view,
                  AutoWebassetsJSMixin) \
            and view_class_name in django_assets.env.get_env():
        return {'full_class_name': view_class_name}
    elif default_js:
        make_bundle(default_js, '.'.join([view_class_name, 'js']),
                    view_class_name)
        return {'full_class_name': view_class_name}




# django-auto-webassets - Automatic webassets javascript bundles for django
# Copyright (C) 2019 Thomas Hartmann <thomas.hartmann@th-ht.de>
#
# This file is part of django-auto-webassets.
#
# django-auto-webassets is free software: you can redistribute it and/or
# modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-auto-webassets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with django-auto-webassets.  If not, see
# <http://www.gnu.org/licenses/>.

import os.path
import uuid

import django.contrib.staticfiles.finders
import django.template
import django.template.library
import django_assets
import django_assets.loaders
import webassets.env
from django.conf import settings

from ..settings import DJANGO_AUTO_WEBASSETS_Settings


def make_bundle(full_webasset_path, output_name, bundle_name, register=True):
    """Make bundle for a javascript file and register it.

    Parameters
    ----------
    full_webasset_path: str
        Full path to the javascript file.
    output_name: str
        Name of the output js file.
    bundle_name: str
        Name of the bundle
    register: bool
        Register the bundle with webassets.

    Returns
    -------
    django_assets.Bundle: The generated bundle.

    """
    these_settings = DJANGO_AUTO_WEBASSETS_Settings()

    if settings.DEBUG or not these_settings.WEBASSETS_OPTIMIZE:
        filters = None
    else:
        filters = 'requirejs'
        os.environ['REQUIREJS_BIN'] = \
            os.path.join(settings.BASE_DIR, these_settings.WEBASSETS_R_JS)
        os.environ[
            'REQUIREJS_CONFIG'] = django.contrib.staticfiles.finders.find(
            these_settings.WEBASSETS_PATH_TO_REQUIREJS_CFG)

    try:
        my_bundle = django_assets.Bundle(
            these_settings.WEBASSETS_PATH_TO_REQUIREJS,
            these_settings.WEBASSETS_PATH_TO_REQUIREJS_CFG, full_webasset_path,
            output=output_name, filters=filters)
        if register:
            django_assets.register(bundle_name, my_bundle)

        return my_bundle
    except webassets.env.RegisterError:
        pass


class WebassetsRjsLoader(django_assets.loaders.DjangoLoader):
    """Webassets Loader for the generated bundles."""

    def _parse(self, filename, contents):
        try:
            t = django.template.Template(str(contents, 'utf-8'))
        except django.template.TemplateSyntaxError:
            pass
        else:
            result = []

            def _recurse_node(node):
                # depending on whether the template tag is added to
                # builtins, or loaded via {% load %}, it will be
                # available in a different module
                # see Django #7430
                if node is not None and \
                        isinstance(node,
                                   django.template.library.InclusionNode) \
                        and node.filename == 'webassets_tag.html':
                    bundle = make_bundle(str(node.args[0].var),
                                         '%s.js' % (str(uuid.uuid4()),),
                                         str(uuid.uuid4()))
                    result.append(bundle)
                for subnode in hasattr(node, 'nodelist') \
                        and node.nodelist or []:
                    _recurse_node(subnode)

            for node in t:
                _recurse_node(node)
            return result



import os
from uuid import uuid4

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

SECRET_KEY = 'CHANGEME'

DEBUG = True

ROOT_URLCONF = 'django_auto_webassets.tests.urls_for_tests'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_assets',
    'django_auto_webassets'
    ]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'django_auto_webassets', 'tests', 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

STATIC_URL = '/static/'

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, 'django_auto_webassets', 'tests', "static"),
]

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'django_assets.finders.AssetsFinder',
)

TEST_RUNNER = 'django_auto_webassets.tests.runner.TestRunner'

WEBASSETS_R_JS = 'django_auto_webassets/tests/node_modules/.bin/r.js'
WEBASSETS_OPTIMIZE = False
ASSETS_DEBUG = False

USE_XVFB = True

ASSETS_MODULES = ['django_auto_webassets.tests.assets.assets']



# django-auto-webassets - Automatic webassets javascript bundles for django
# Copyright (C) 2019 Thomas Hartmann <thomas.hartmann@th-ht.de>
#
# This file is part of django-auto-webassets.
#
# django-auto-webassets is free software: you can redistribute it and/or
# modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-auto-webassets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with django-auto-webassets.  If not, see
# <http://www.gnu.org/licenses/>.

from django.core.management import call_command
from .helpers.functions import product_dict
import itertools
import pytest

ALL_SETTINGS = {
    'ASSETS_DEBUG': (True, False),
    'WEBASSETS_OPTIMIZE': (True, False),
    'WEBASSETS_SCRAMBLE_OUT_JS': (True, False),
}

ALL_SETTINGS_PRODUCT = list(product_dict(**ALL_SETTINGS))
USE_BUNDLES = [True, False]

ALL_PARAMS_PRODUCT = itertools.product(ALL_SETTINGS_PRODUCT, USE_BUNDLES)


def _idfn(val):
    if isinstance(val, dict):
        val = str(val)
    return val


@pytest.mark.parametrize('override_settings, make_bundle',
                         ALL_PARAMS_PRODUCT, ids=_idfn)
def test_functional(selenium, live_server, settings,
                    override_settings, make_bundle):
    """Functional test for all possible config values."""
    for key, value in override_settings.items():
        setattr(settings, key, value)
    call_command('assets', 'clean')
    if make_bundle:
        call_command('webassetsrequirejs')

    selenium.open(live_server.url + '/test_functional/')
    selenium.assert_text('Changed', '.test_class')
    selenium.open(live_server.url + '/test_mixin/')
    selenium.assert_text('This is new', '.test_class')
    selenium.open(live_server.url + '/test_other/test_functional/')
    selenium.assert_text('Changed', '.test_class')
    selenium.open(live_server.url + '/test_other/test_mixin/')
    selenium.assert_text('This is new', '.test_class')
    selenium.open(live_server.url + '/test_namespace/test_functional/')
    selenium.assert_text('Changed', '.test_class')
    selenium.open(live_server.url + '/test_namespace/test_mixin/')
    selenium.assert_text('This is new', '.test_class')


ALL_PARAMS_PRODUCT = itertools.product(ALL_SETTINGS_PRODUCT, USE_BUNDLES)
@pytest.mark.parametrize('override_settings, make_bundle',
                         ALL_PARAMS_PRODUCT, ids=_idfn)
def test_css(selenium, live_server, settings,
             override_settings, make_bundle):
    """Test for CSS bundles."""
    for key, value in override_settings.items():
        setattr(settings, key, value)
    call_command('assets', 'clean')
    if make_bundle:
        call_command('webassetsrequirejs')

    selenium.open(live_server.url + '/test_css/')
    selenium.assert_text('Hello', '.test_class')
